<?php

namespace Bss\CfFix\Preference\SalesOrder\Model\ResourceModel\Grid;

use Bss\SalesRep\Helper\Data as HelperData;
use Magento\Framework\Data\Collection\Db\FetchStrategyInterface as FetchStrategy;
use Magento\Framework\Data\Collection\EntityFactoryInterface as EntityFactory;
use Magento\Framework\Event\ManagerInterface as EventManager;
use Psr\Log\LoggerInterface as Logger;

class Collection extends \Magento\Sales\Model\ResourceModel\Order\Grid\Collection
{
    /**
     * @var HelperData
     */
    protected $helperData;

    /**
     * Collection constructor.
     * @param EntityFactory $entityFactory
     * @param Logger $logger
     * @param HelperData $helperData
     * @param FetchStrategy $fetchStrategy
     * @param EventManager $eventManager
     * @param string $mainTable
     * @param string $resourceModel
     */
    public function __construct(
        EntityFactory $entityFactory,
        Logger $logger,
        HelperData $helperData,
        FetchStrategy $fetchStrategy,
        EventManager $eventManager,
        $mainTable = 'sales_order_grid',
        $resourceModel = \Magento\Sales\Model\ResourceModel\Order::class
    ) {
        $this->helperData = $helperData;
        parent::__construct(
            $entityFactory,
            $logger,
            $fetchStrategy,
            $eventManager,
            $mainTable,
            $resourceModel
        );
    }

    /**
     * @inheritDoc
     */
    protected function _initSelect()
    {
        $columns = $this->getFields();
        $this->addFilterToMap('sales_rep', 'admin_user.username');
        $this->addFilterToMap('store_id', 'main_table.store_id');
        parent::_initSelect();

        // Sales Rep
        $this->getSelect()->joinLeft(
            ['bss_sales_rep_order' => $this->getTable('bss_sales_rep_order')],
            'main_table.entity_id = bss_sales_rep_order.order_id',
            ['user_id' => 'bss_sales_rep_order.user_id']
        )->joinLeft(
            ['admin_user' => $this->getTable('admin_user')],
            'bss_sales_rep_order.user_id = admin_user.user_id',
            ['sales_rep' => 'admin_user.username']
        );
        if ($this->helperData->isEnableCompanyAccount()) {
            $this->getSelect()->joinLeft(
                ['bss_sub_user_order' => $this->getTable('bss_sub_user_order')],
                'bss_sales_rep_order.order_id = bss_sub_user_order.order_id',
                ['sub_id' => 'bss_sub_user_order.sub_id']
            )
                ->joinLeft(
                    ['bss_sub_user' => $this->getTable('bss_sub_user')],
                    'bss_sub_user.sub_id = bss_sub_user_order.sub_id',
                    ['sub_name' => 'bss_sub_user.sub_name']
                )->distinct(true);
        }

        // Checkout Custom Fields
        $this->getSelect()->joinLeft(
            ['grid_custom_field' => $this->getTable('bss_checkout_attribute_order_grid_view')],
            'main_table.increment_id = grid_custom_field.incrementId AND main_table.store_id = grid_custom_field.store_id',
            $columns
        );
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getFields()
    {
        $columns = [];
        $fields = $this->getConnection()->describeTable($this->getTable('bss_checkout_attribute_order_grid_view'));
        foreach ($fields as $field) {
            $columns[] = $field['COLUMN_NAME'];
        }
        return $columns;
    }
}